from django.db import models

# Create your models here.
from wagtail.snippets.models import register_snippet
from wagtailmodelchooser import register_model_chooser
from modelcluster.fields import ParentalKey
from modelcluster.models import ClusterableModel
from django_extensions.db.fields import AutoSlugField
# from modelcluster.contrib.taggit import ClusterTaggableManager

from wagtail_color_panel.fields import ColorField
from wagtail_color_panel.edit_handlers import NativeColorPanel
from wagtail.fields import RichTextField
from wagtail.admin.panels import FieldPanel, InlinePanel, MultiFieldPanel, FieldRowPanel
from wagtail.models import Page, Orderable

from wagtail.contrib.routable_page.models import RoutablePageMixin, path, re_path
from wagtail.fields import StreamField
from hitcount.models import HitCountMixin
from wagtail.admin.panels import TabbedInterface, TitleFieldPanel, ObjectList
from wagtail.search import index
from .blocks import *
from .panels import SubcategoryChooser
from wagtailmetadata.models import MetadataPageMixin


class TagColor(models.Model):
    bgColor = ColorField(
        default="#FFFFFF", verbose_name="BG Color", blank=True, null=True)
    fgColor = ColorField(
        default="#000000", verbose_name="FG Color", blank=True, null=True)
    hoverBgColor = ColorField(
        default="#FFFFFF", verbose_name="Hover BG Color", blank=True, null=True)
    hoverFgColor = ColorField(
        default="#000000", verbose_name="Hover FG Color", blank=True, null=True)

    class Meta:
        abstract = True


@register_snippet
@register_model_chooser
class Category(ClusterableModel, TagColor):
    catName = models.CharField(max_length=250, verbose_name="Category Name")
    catSlug = AutoSlugField(populate_from="catName",
                            editable=True, verbose_name="Slug")
    catImg = models.ForeignKey(
        'wagtailimages.Image', on_delete=models.CASCADE, related_name='+', blank=True, null=True
    )
    catImgText = models.CharField(blank=True, null=True,
                                  max_length=255,
                                  help_text="Get icons from https://forkaweso.me/Fork-Awesome/icons/"
                                  )

    catShortDesc = RichTextField(verbose_name="Short Description",
                                 null=True, blank=True)
    order = models.IntegerField(unique=True, null=True, blank=True)
    isLive = models.BooleanField(default=True, verbose_name="Active")
    service = models.BooleanField(default=False, verbose_name="Service")

    panels = [
        MultiFieldPanel([
            FieldRowPanel([
                FieldPanel('catName'),
                FieldPanel('catSlug'),

            ]),
            FieldRowPanel([
                NativeColorPanel('bgColor'),
                NativeColorPanel('fgColor'),

            ]),
            FieldRowPanel([
                FieldPanel('catImg', heading="Category Image"),
                FieldPanel('order'),
                FieldPanel('isLive'),
                FieldPanel('service'),


            ]),
            FieldPanel('catImgText', heading="Category Text Image"),
            FieldPanel('catShortDesc'),
        ]),

        InlinePanel('sub_categories', label="Sub Category")


    ]

    def __str__(self):
        return str(self.catName)

    class Meta:
        verbose_name_plural = "Category"
        ordering = ['order']


class SubCategory(Orderable, TagColor):
    catName = ParentalKey(Category, related_name="sub_categories")
    scName = models.CharField(
        max_length=250, blank=True, null=True, verbose_name="Name")
    scSlug = AutoSlugField(populate_from="scName",
                           editable=True, verbose_name="Slug")
    scImg = models.ForeignKey(
        'wagtailimages.Image', on_delete=models.CASCADE, related_name='+', blank=True, null=True
    )
    scImgText = models.CharField(blank=True, null=True,
                                 max_length=255,
                                 help_text="Get icons from https://forkaweso.me/Fork-Awesome/icons/"
                                 )
    scDesc = RichTextField(verbose_name="Description",
                           null=True, blank=True)
    service = models.BooleanField(default=True, verbose_name="Service")

    # isLive = models.BooleanField(default=True, verbose_name="Active")

    panels = [
        # FieldPanel('service'),
        MultiFieldPanel([
            FieldRowPanel([
                FieldPanel('scName'),
                FieldPanel('scSlug'),
            ]),
            FieldRowPanel([
                NativeColorPanel('bgColor'),
                NativeColorPanel('fgColor'),

            ]),
            FieldPanel('scImg', heading="Image"),
            FieldPanel('scImgText', heading="Text Image"),
        ]),
        FieldPanel('service'),

        FieldPanel('scDesc'),
    ]

    def __str__(self):
        return self.scName

    class Meta:
        verbose_name_plural = "Sub Category"
        verbose_name = "Sub Category"
        ordering = ['-id']


class ServiceIndexPage(MetadataPageMixin,RoutablePageMixin, Page):
    intro = RichTextField(blank=True)
    banner = StreamField([
        ('banner', CarouselBlock()),
    ], use_json_field=True, blank=True, null=True, max_num=1)
    # template = "home/home_page.html"

    parent_page_type = [
        'Core.WebPages',
    ]
    subpage_types = [
        'Service.ServicePage',
        'Service.ProductPage',
    ]

    content_panels = Page.content_panels + [
        FieldPanel('intro'),
        FieldPanel('banner')
    ]

    # def get_children(self):
    #     qs = super().get_children()
    #     # qs = BlogPage.objects.all()
    #     qs = qs.order_by('-last_published_at')
    #     return qs

    def get_child_blogs(self):
        qs = BlogPage.objects.all()
        qs = qs.order_by('-last_published_at')
        return qs

    def get_recent_blogs(self):
        max_count = 5  # max count for displaying post
        return BlogPage.objects.all().order_by('-first_published_at')[:max_count]

    def get_trending_blogs(self):
        max_count = 5  # max count for displaying post
        return BlogPage.objects.all().order_by('-blPoViewCount')[:max_count]

    def get_tags(self):
        return BlogTag.objects.all().order_by('-id')

    def get_categories(self):
        return CategoryPage.objects.filter(type="Category").order_by('-id')

    @path('tag/<tag>/')
    def post_by_tag(self, request, tag, *args, **kwargs):
        self.posts = BlogPage.objects.live().filter(tags__slug=tag)
        return self.render(request, context_overrides={
            'title': "Blogs for <b>"+tag+"</b>",
            'posts': self.posts,
        })

    @path('category/<str:slug>/')
    def post_by_category(self, request, slug, *args, **kwargs):
        # events = BlogPage.objects.live().filter(Q(product__coProdCat__scSlug=slug) |
        #                                         Q(product__coProdCat__catName__catSlug=slug))
        blogsList = []
        for blog in BlogPage.objects.all():
            for blCat in blog.blCategory:
                if blCat.value.slug == slug:
                    blogsList.append(blog)
        return self.render(request, context_overrides={
            'title': "Blogs for <b>" + slug+"</b>",
            'posts': blogsList,
        })

    @path('<int:year>/<int:month>/')
    def post_by_year_month(self, request, year=None, month=None, *args, **kwargs):
        events = BlogPage.objects.live().filter(
            first_published_at__year=year, first_published_at__month=month)
        # return self.render(request)
        print("Blogs=", events)
        return self.render(request, context_overrides={
            'title': "Blogs for <b>"+calendar.month_name[month]+", "+str(year)+"</b>",
            'posts': events,
        })

from wagtail.api import APIField

class ServicePage(MetadataPageMixin,Page, HitCountMixin):
    titleImg = models.ForeignKey(
        'wagtailimages.Image', on_delete=models.PROTECT, related_name='+', blank=True, null=True, verbose_name="Title Image"
    )
    shortDesc = RichTextField(blank=True, null=True,
                              verbose_name="Short Description")
    category = models.ForeignKey(
        SubCategory, on_delete=models.SET_NULL, null=True, related_name="services")

    body = StreamField([
        ('section', SectionBlock()),
        ('row', RowBlock()),
    ], use_json_field=True, blank=True, null=True)
    banner = StreamField([
        ('banner', CarouselBlock()),
    ], use_json_field=True, blank=True, null=True, max_num=1)

    showFooter = models.BooleanField(
        blank=True, null=True, default=False, verbose_name="Show Footer")

    api_fields = [
        APIField('body'),
        APIField('titleImg'),
        APIField('shortDesc'),
        APIField('category'),  
    ]
    
    content_panels = Page.content_panels + [
        # FieldPanel('category', heading="Category"),
        SubcategoryChooser(
            "category",
            category=Category,
            subcategory=SubCategory,
            subcategory_related_name="sub_categories",
            search_text="Search Subcategories"
        ),
        MultiFieldPanel([
            FieldRowPanel([
                FieldPanel('titleImg'),
            ])
        ]),
        FieldPanel('shortDesc'),
        FieldPanel('body'),
    ]
    promote_panels = Page.promote_panels + [
        FieldPanel('showFooter'),
    ]
    banner_panels = [
        FieldPanel('banner', heading="Banner")
    ]
    edit_handler = TabbedInterface([
        ObjectList(content_panels, heading='Content'),
        ObjectList(promote_panels, heading='Promote'),
        ObjectList(banner_panels, heading='Banner'),
    ])
    search_fields = Page.search_fields + [  # Inherit search_fields from Page
        index.SearchField('title'),
        index.SearchField('category'),
        index.SearchField('body'),
        index.FilterField('title'),
        index.FilterField('category'),
        index.FilterField('body'),
    ]

    def __str__(self):
        return str(self.title)

    def get_absolute_url(self):
        return f'{self.url}'

    class Meta:
        verbose_name = "Service Page"


class ProductPage(MetadataPageMixin,Page, HitCountMixin):
    titleImg = models.ForeignKey(
        'wagtailimages.Image', on_delete=models.PROTECT, related_name='+', blank=True, null=True, verbose_name="Title Image"
    )
    shortDesc = RichTextField(blank=True, null=True,
                              verbose_name="Short Description")
    category = models.ForeignKey(
        SubCategory, on_delete=models.SET_NULL, null=True, related_name="products")

    body = StreamField([
        ('section', SectionBlock()),
        ('row', RowBlock()),
    ], use_json_field=True, blank=True, null=True)
    banner = StreamField([
        ('banner', CarouselBlock()),
    ], use_json_field=True, blank=True, null=True, max_num=1)

    showFooter = models.BooleanField(
        blank=True, null=True, default=False, verbose_name="Show Footer")

    content_panels = Page.content_panels + [
        FieldPanel('category', heading="Category"),
        MultiFieldPanel([
            FieldRowPanel([
                FieldPanel('titleImg'),
            ])
        ]),
        FieldPanel('shortDesc'),
        FieldPanel('body'),
    ]
    promote_panels = Page.promote_panels + [
        FieldPanel('showFooter'),
    ]
    banner_panels = [
        FieldPanel('banner', heading="Banner")
    ]
    edit_handler = TabbedInterface([
        ObjectList(content_panels, heading='Content'),
        ObjectList(promote_panels, heading='Promote'),
        ObjectList(banner_panels, heading='Banner'),
    ])
    search_fields = Page.search_fields + [  # Inherit search_fields from Page
        index.SearchField('title'),
        index.SearchField('category'),
        index.SearchField('body'),
        index.FilterField('title'),
        index.FilterField('category'),
        index.FilterField('body'),
    ]

    def __str__(self):
        return str(self.title)

    class Meta:
        verbose_name = "Product Page"
